"""
Fitting modules for resonator data.

This package provides methods for fitting resonator data to extract
parameters such as resonance frequency, quality factors, etc.
"""

# Import specific fitting methods
from resonfit.fitting.methods import DCMFitter
